searchData={"items":[{"type":"module","title":"FitDecoder","doc":"A wrapper for the C++ FIT file decoder NIF.\n\nThis module provides functionality to decode FIT (Flexible and Interoperable Data Transfer) files,\nwhich are commonly used by fitness devices and applications to store workout and activity data.","ref":"FitDecoder.html"},{"type":"module","title":"Examples - FitDecoder","doc":"# Decode an empty binary (returns empty list)\n    iex> FitDecoder.decode_fit_file(<<>>)\n    []\n\n    # Decode invalid binary data (returns error atom)\n    iex> FitDecoder.decode_fit_file(<<1, 2, 3, 4>>)\n    :error_integrity_check_failed\n\n    # Decode a valid FIT file binary would return a list of maps:\n    # [%{timestamp: 1096265262, distance: 0.0, heart_rate: 95}, ...]","ref":"FitDecoder.html#module-examples"},{"type":"function","title":"FitDecoder.decode_and_analyze/1","doc":"Convenience function that combines file reading, decoding, and info extraction.\n\nThis function implements the complete workflow: read file → decode → extract info.","ref":"FitDecoder.html#decode_and_analyze/1"},{"type":"function","title":"Parameters - FitDecoder.decode_and_analyze/1","doc":"* `file_path` - Path to a FIT file on disk","ref":"FitDecoder.html#decode_and_analyze/1-parameters"},{"type":"function","title":"Returns - FitDecoder.decode_and_analyze/1","doc":"* `{:ok, {activity_info, records}}` - Activity info and full record list\n  * `{:error, reason}` - If file reading, decoding, or parsing fails\n  * Error atoms from the decoder (e.g., `:error_integrity_check_failed`)","ref":"FitDecoder.html#decode_and_analyze/1-returns"},{"type":"function","title":"Examples - FitDecoder.decode_and_analyze/1","doc":"iex> FitDecoder.decode_and_analyze(\"/nonexistent/file.fit\")\n    {:error, :enoent}","ref":"FitDecoder.html#decode_and_analyze/1-examples"},{"type":"function","title":"FitDecoder.decode_fit_file/1","doc":"Decodes a FIT file binary and returns a list of maps, where each map\nrepresents a \"Record\" message containing sensor data.","ref":"FitDecoder.html#decode_fit_file/1"},{"type":"function","title":"Parameters - FitDecoder.decode_fit_file/1","doc":"* `binary` - A binary containing FIT file data","ref":"FitDecoder.html#decode_fit_file/1-parameters"},{"type":"function","title":"Returns - FitDecoder.decode_fit_file/1","doc":"* A list of maps containing decoded record data on success\n  * An error atom (`:error_integrity_check_failed`, `:error_sdk_exception`) on failure\n  * An empty list if the file contains no record data","ref":"FitDecoder.html#decode_fit_file/1-returns"},{"type":"function","title":"Record Fields - FitDecoder.decode_fit_file/1","doc":"Each record map may contain the following fields (only present if valid data exists):\n\n  * `:timestamp` - Unix timestamp (always present in valid records)\n  * `:distance` - Distance in meters (float)\n  * `:heart_rate` - Heart rate in beats per minute (integer)\n  * `:altitude` - Altitude in meters (float)","ref":"FitDecoder.html#decode_fit_file/1-record-fields"},{"type":"function","title":"Examples - FitDecoder.decode_fit_file/1","doc":"iex> FitDecoder.decode_fit_file(<<>>)\n    []\n\n    iex> FitDecoder.decode_fit_file(<<1, 2, 3, 4>>)\n    :error_integrity_check_failed","ref":"FitDecoder.html#decode_fit_file/1-examples"},{"type":"function","title":"FitDecoder.decode_fit_file_from_path/1","doc":"Decodes a FIT file from a file path and returns the same data as `decode_fit_file/1`.","ref":"FitDecoder.html#decode_fit_file_from_path/1"},{"type":"function","title":"Parameters - FitDecoder.decode_fit_file_from_path/1","doc":"* `file_path` - Path to a FIT file on disk","ref":"FitDecoder.html#decode_fit_file_from_path/1-parameters"},{"type":"function","title":"Returns - FitDecoder.decode_fit_file_from_path/1","doc":"* Same as `decode_fit_file/1` but reads from file path\n  * `{:error, reason}` if file cannot be read","ref":"FitDecoder.html#decode_fit_file_from_path/1-returns"},{"type":"function","title":"Examples - FitDecoder.decode_fit_file_from_path/1","doc":"iex> FitDecoder.decode_fit_file_from_path(\"/nonexistent/file.fit\")\n    {:error, :enoent}","ref":"FitDecoder.html#decode_fit_file_from_path/1-examples"},{"type":"function","title":"FitDecoder.get_activity_date/1","doc":"Gets the activity date from decoded FIT file records.\n\nReturns the date (as a `Date` struct) when the activity started based on the\nearliest timestamp in the records.","ref":"FitDecoder.html#get_activity_date/1"},{"type":"function","title":"Parameters - FitDecoder.get_activity_date/1","doc":"* `records` - List of decoded FIT records from `decode_fit_file/1`","ref":"FitDecoder.html#get_activity_date/1-parameters"},{"type":"function","title":"Returns - FitDecoder.get_activity_date/1","doc":"* `{:ok, %Date{}}` - The date when the activity started\n  * `{:error, :no_records}` - If the records list is empty\n  * `{:error, :invalid_records}` - If records don't contain valid timestamps","ref":"FitDecoder.html#get_activity_date/1-returns"},{"type":"function","title":"Examples - FitDecoder.get_activity_date/1","doc":"iex> records = [%{timestamp: 1745940695}]\n    iex> FitDecoder.get_activity_date(records)\n    {:ok, ~D[2025-04-29]}","ref":"FitDecoder.html#get_activity_date/1-examples"},{"type":"function","title":"FitDecoder.get_activity_duration/1","doc":"Gets the activity duration from decoded FIT file records.\n\nReturns the duration of the activity in seconds based on the difference\nbetween the earliest and latest timestamps.","ref":"FitDecoder.html#get_activity_duration/1"},{"type":"function","title":"Parameters - FitDecoder.get_activity_duration/1","doc":"* `records` - List of decoded FIT records from `decode_fit_file/1`","ref":"FitDecoder.html#get_activity_duration/1-parameters"},{"type":"function","title":"Returns - FitDecoder.get_activity_duration/1","doc":"* `{:ok, duration_seconds}` - Duration in seconds (integer)\n  * `{:error, :no_records}` - If the records list is empty\n  * `{:error, :invalid_records}` - If records don't contain valid timestamps\n  * `{:error, :insufficient_data}` - If there's only one record","ref":"FitDecoder.html#get_activity_duration/1-returns"},{"type":"function","title":"Examples - FitDecoder.get_activity_duration/1","doc":"iex> records = [%{timestamp: 1096265262}, %{timestamp: 1096265322}]\n    iex> FitDecoder.get_activity_duration(records)\n    {:ok, 60}","ref":"FitDecoder.html#get_activity_duration/1-examples"},{"type":"function","title":"FitDecoder.get_activity_info/1","doc":"Gets comprehensive activity information from decoded FIT file records.\n\nThis is a convenience function that extracts the most commonly needed\ninformation from an activity in a single call.","ref":"FitDecoder.html#get_activity_info/1"},{"type":"function","title":"Parameters - FitDecoder.get_activity_info/1","doc":"* `records` - List of decoded FIT records from `decode_fit_file/1`","ref":"FitDecoder.html#get_activity_info/1-parameters"},{"type":"function","title":"Returns - FitDecoder.get_activity_info/1","doc":"* `{:ok, activity_info}` - A map containing activity information\n  * `{:error, reason}` - If records are invalid or empty\n\nThe activity_info map contains:\n  * `:date` - Date when activity started (`Date` struct)\n  * `:duration_seconds` - Duration in seconds (integer)\n  * `:total_distance` - Total distance in meters (float, may be nil)\n  * `:record_count` - Number of data records (integer)\n  * `:has_heart_rate` - Whether heart rate data is present (boolean)\n  * `:has_altitude` - Whether altitude data is present (boolean)","ref":"FitDecoder.html#get_activity_info/1-returns"},{"type":"function","title":"Examples - FitDecoder.get_activity_info/1","doc":"iex> records = [%{timestamp: 1727321178, distance: 12543.2, heart_rate: 95}, %{timestamp: 1727321238, distance: 12543.2, heart_rate: 98}]\n    iex> FitDecoder.get_activity_info(records)\n    {:ok, %{\n      date: ~D[2024-09-26],\n      duration_seconds: 60,\n      total_distance: 12543.2,\n      record_count: 2,\n      has_heart_rate: true,\n      has_altitude: false\n    }}","ref":"FitDecoder.html#get_activity_info/1-examples"},{"type":"module","title":"FitDecoder.FieldDemo","doc":"Demo module to load a FIT file and show what data is present.\n\nUsage in IEx:\n    iex> FitDecoder.FieldDemo.run()\n    iex> FitDecoder.FieldDemo.run(\"/path/to/your/file.fit\")","ref":"FitDecoder.FieldDemo.html"},{"type":"function","title":"FitDecoder.FieldDemo.field_stats/2","doc":"Get field statistics for a specific field","ref":"FitDecoder.FieldDemo.html#field_stats/2"},{"type":"function","title":"FitDecoder.FieldDemo.run/1","doc":"","ref":"FitDecoder.FieldDemo.html#run/1"},{"type":"function","title":"FitDecoder.FieldDemo.show_fields/1","doc":"Convenience function to show just the available fields in a FIT file","ref":"FitDecoder.FieldDemo.html#show_fields/1"},{"type":"module","title":"FitDecoder.NIF","doc":"","ref":"FitDecoder.NIF.html"},{"type":"function","title":"FitDecoder.NIF.decode_fit_file/1","doc":"","ref":"FitDecoder.NIF.html#decode_fit_file/1"},{"type":"function","title":"FitDecoder.NIF.load_nif/0","doc":"","ref":"FitDecoder.NIF.html#load_nif/0"},{"type":"extras","title":"FitDecoder","doc":"# FitDecoder\n\nFitDecoder is a high-performance Elixir library for decoding Garmin .fit files. It uses a C++ NIF (Native Implemented Function) to wrap the official Garmin FIT C++ SDK, providing a fast and efficient way to extract data from activity files directly within the Elixir ecosystem.\n\nThe primary goal of this library is to parse the \"Record\" messages from a FIT file, which contain moment-by-moment sensor data like GPS coordinates, altitude, heart rate, power, cadence, and many more fields covering everything from basic activity tracking to advanced physiological metrics.","ref":"readme.html"},{"type":"extras","title":"Requirements - FitDecoder","doc":"To build and use this library, you will need the following installed on your system (macOS or Linux):\n\n- Elixir (~> 1.15 or newer)\n- Erlang/OTP (~> 25.0 or newer)\n- A C++ compiler (e.g., g++ or clang++)\n- make","ref":"readme.html#requirements"},{"type":"extras","title":"Build Instructions - FitDecoder","doc":"","ref":"readme.html#build-instructions"},{"type":"extras","title":"1. Clone the Repository - FitDecoder","doc":"```bash\ngit clone  \ncd fit_decoder\n```","ref":"readme.html#1-clone-the-repository"},{"type":"extras","title":"2. Download the Garmin FIT SDK - FitDecoder","doc":"1. Go to the [FIT SDK download page](https://developer.garmin.com/fit/download/) and download the latest version\n2. Unzip the downloaded file\n3. Locate the `cpp` directory inside the unzipped folder\n4. Copy the entire `cpp` directory into this project's `c_src/fit_sdk/` directory\n\nThe final path should be `c_src/fit_sdk/cpp`.","ref":"readme.html#2-download-the-garmin-fit-sdk"},{"type":"extras","title":"3. Fetch Dependencies and Compile - FitDecoder","doc":"Run the following commands to fetch the Elixir dependencies and compile the project. This will also compile the C++ NIF.\n\n```bash\nmix deps.get\nmix compile\n```\n\nIf the compilation is successful, you are ready to use the library.","ref":"readme.html#3-fetch-dependencies-and-compile"},{"type":"extras","title":"Usage - FitDecoder","doc":"The library provides both low-level decoding functions and high-level helper functions for common workflows. Most application developers will want to use the helper functions for a streamlined experience.","ref":"readme.html#usage"},{"type":"extras","title":"Quick Demo - FitDecoder","doc":"To quickly see what data is available in your FIT files, use the built-in demo:\n\n```elixir\n# Start IEx\niex -S mix\n\n# Run the demo (will look for test.fit in common locations)\nFitDecoder.FieldDemo.run()\n\n# Or specify a specific file\nFitDecoder.FieldDemo.run(\"/path/to/your/activity.fit\")\n\n# Just show available fields\nFitDecoder.FieldDemo.show_fields(\"/path/to/your/activity.fit\")\n\n# Get statistics for a specific field\nFitDecoder.FieldDemo.field_stats(:heart_rate, \"/path/to/your/activity.fit\")\n```","ref":"readme.html#quick-demo"},{"type":"extras","title":"Recommended Workflow (Helper Functions) - FitDecoder","doc":"For most applications, use the helper functions that provide a complete workflow:\n\n```elixir\n# Complete workflow in one function call\ncase FitDecoder.decode_and_analyze(\"/path/to/activity.fit\") do\n  {:ok, {activity_info, records}} ->\n    # Get comprehensive activity information\n    IO.puts(\"Date: #{activity_info.date}\")\n    IO.puts(\"Duration: #{activity_info.duration_seconds} seconds\") \n    IO.puts(\"Distance: #{activity_info.total_distance} meters\")\n    IO.puts(\"Records: #{activity_info.record_count}\")\n    IO.puts(\"Has heart rate: #{activity_info.has_heart_rate}\")\n    \n    # Access individual records for detailed analysis\n    Enum.each(records, fn record ->\n      if Map.has_key?(record, :heart_rate) do\n        IO.puts(\"HR: #{record.heart_rate} bpm\")\n      end\n    end)\n    \n  {:error, reason} ->\n    IO.puts(\"Failed to process FIT file: #{reason}\")\n    \n  error_atom when is_atom(error_atom) ->\n    IO.puts(\"FIT decoding error: #{error_atom}\")\nend\n```","ref":"readme.html#recommended-workflow-helper-functions"},{"type":"extras","title":"Step-by-Step Workflow - FitDecoder","doc":"If you prefer more control, use the individual helper functions:\n\n```elixir\n# Method 1: From file path\ncase FitDecoder.decode_fit_file_from_path(\"/path/to/activity.fit\") do\n  records when is_list(records) ->\n    # Get activity date\n    {:ok, date} = FitDecoder.get_activity_date(records)\n    IO.puts(\"Activity date: #{date}\")\n    \n    # Get activity duration\n    {:ok, duration} = FitDecoder.get_activity_duration(records)\n    IO.puts(\"Duration: #{duration} seconds\")\n    \n    # Get comprehensive info\n    {:ok, info} = FitDecoder.get_activity_info(records)\n    IO.inspect(info)\n    \n  error -> IO.puts(\"Error: #{error}\")\nend\n\n# Method 2: From binary data\n{:ok, fit_binary} = File.read(\"/path/to/activity.fit\")\nrecords = FitDecoder.decode_fit_file(fit_binary)\n{:ok, activity_info} = FitDecoder.get_activity_info(records)\n```","ref":"readme.html#step-by-step-workflow"},{"type":"extras","title":"Low-Level Usage - FitDecoder","doc":"For direct access to the decoder:\n\n```elixir\n# Read and decode manually\n{:ok, fit_binary} = File.read(\"/path/to/activity.fit\")\nrecords = FitDecoder.decode_fit_file(fit_binary)\n\nIO.puts(\"Found #{length(records)} records.\")\nIO.inspect(hd(records), label: \"First Record\")\n```","ref":"readme.html#low-level-usage"},{"type":"extras","title":"Advanced Usage - FitDecoder","doc":"Access any of the 96+ available fields:\n\n```elixir\n# Basic activity data\nEnum.each(records, fn record ->\n  IO.puts(\"Time: #{record.timestamp}\")\n  if Map.has_key?(record, :distance), do: IO.puts(\"Distance: #{record.distance}m\")\n  if Map.has_key?(record, :heart_rate), do: IO.puts(\"HR: #{record.heart_rate} bpm\")\nend)\n\n# Power analysis (cycling)\npower_records = Enum.filter(records, &Map.has_key?(&1, :power))\nif length(power_records) > 0 do\n  avg_power = power_records |> Enum.map(& &1.power) |> Enum.sum() |> div(length(power_records))\n  IO.puts(\"Average power: #{avg_power}W\")\nend\n\n# GPS tracking\ngps_points = records\n|> Enum.filter(fn record ->\n  Map.has_key?(record, :position_lat) && Map.has_key?(record, :position_long)\nend)\n|> Enum.map(fn record ->\n  %{\n    lat: record.position_lat / :math.pow(2, 31) * 180,\n    lng: record.position_long / :math.pow(2, 31) * 180,\n    altitude: Map.get(record, :altitude),\n    timestamp: record.timestamp\n  }\nend)\n\n# Running dynamics\nrunning_data = records\n|> Enum.filter(&Map.has_key?(&1, :vertical_oscillation))\n|> Enum.map(fn record ->\n  %{\n    cadence: Map.get(record, :cadence),\n    vertical_oscillation: record.vertical_oscillation,\n    stance_time: Map.get(record, :stance_time),\n    step_length: Map.get(record, :step_length)\n  }\nend)\n```","ref":"readme.html#advanced-usage"},{"type":"extras","title":"Return Value - FitDecoder","doc":"The `decode_fit_file/1` function returns a list of maps. Each map represents a single \"Record\" message from the FIT file and can contain any of **96+ available fields** depending on your device and activity type.","ref":"readme.html#return-value"},{"type":"extras","title":"Core Fields - FitDecoder","doc":"Always present when valid:\n- `:timestamp` - (Integer) The FIT timestamp for the data point","ref":"readme.html#core-fields"},{"type":"extras","title":"Common Fields - FitDecoder","doc":"- `:altitude` - (Float) Altitude in meters\n- `:distance` - (Float) Total distance traveled in meters\n- `:heart_rate` - (Integer) Heart rate in beats per minute\n- `:speed` - (Float) Speed in meters per second\n- `:cadence` - (Integer) Cadence in RPM\n- `:power` - (Integer) Power in watts\n- `:position_lat` - (Integer) Latitude in semicircles\n- `:position_long` - (Integer) Longitude in semicircles","ref":"readme.html#common-fields"},{"type":"extras","title":"Advanced Fields - FitDecoder","doc":"- **Running Dynamics**: `:vertical_oscillation`, `:stance_time`, `:step_length`\n- **Cycling Metrics**: `:left_torque_effectiveness`, `:pedal_smoothness`, `:left_right_balance`\n- **Physiological**: `:respiration_rate`, `:current_stress`, `:core_temperature`\n- **E-bike**: `:battery_soc`, `:motor_power`, `:ebike_assist_mode`\n- **Diving**: `:depth`, `:absolute_pressure`, `:air_time_remaining`\n- **Blood/Oxygen**: `:saturated_hemoglobin_percent`, `:total_hemoglobin_conc`","ref":"readme.html#advanced-fields"},{"type":"extras","title":"Example Records - FitDecoder","doc":"Basic cycling record:\n```elixir\n%{\n  timestamp: 978318654,\n  altitude: 153.2,\n  distance: 5.96,\n  heart_rate: 111,\n  speed: 4.2,\n  power: 185,\n  cadence: 87\n}\n```\n\nAdvanced running record:\n```elixir\n%{\n  timestamp: 978318655,\n  distance: 1250.5,\n  heart_rate: 145,\n  speed: 3.8,\n  vertical_oscillation: 8.2,\n  stance_time: 245.0,\n  step_length: 1.45,\n  cadence: 180\n}\n```\n\nGPS-enabled record:\n```elixir\n%{\n  timestamp: 978318656,\n  position_lat: 407745893,  # Convert: lat_degrees = value / 2^31 * 180\n  position_long: -1221066674,\n  altitude: 156.8,\n  enhanced_speed: 2.1,\n  gps_accuracy: 3\n}\n```\n\nSee [FIELDS.md](FIELDS.md) for complete field documentation.","ref":"readme.html#example-records"},{"type":"extras","title":"Field Availability - FitDecoder","doc":"Not all fields will be present in every FIT file. Field availability depends on:\n\n1. **Device capabilities** - Only devices with specific sensors can record certain metrics\n2. **Activity type** - Swimming fields won't appear in cycling activities  \n3. **Recording settings** - Some fields may be disabled to save battery/storage\n4. **FIT file version** - Newer fields may not exist in older files\n\nAlways check for field presence using `Map.has_key?/2` before accessing values.","ref":"readme.html#field-availability"},{"type":"extras","title":"Supported Activity Types - FitDecoder","doc":"This decoder supports FIT files from various devices and activities:\n\n- **Cycling**: Road, mountain, indoor, e-bike\n- **Running**: Road, trail, treadmill, track\n- **Swimming**: Pool, open water\n- **Fitness**: Gym workouts, strength training\n- **Outdoor Activities**: Hiking, skiing, diving\n- **Multi-sport**: Triathlon, adventure racing","ref":"readme.html#supported-activity-types"},{"type":"extras","title":"Helper Functions - FitDecoder","doc":"The library provides several helper functions to make common workflows easier:","ref":"readme.html#helper-functions"},{"type":"extras","title":"Core Functions - FitDecoder","doc":"- `FitDecoder.decode_fit_file_from_path/1` - Decode directly from file path\n- `FitDecoder.get_activity_date/1` - Extract activity start date\n- `FitDecoder.get_activity_duration/1` - Calculate activity duration in seconds\n- `FitDecoder.get_activity_info/1` - Get comprehensive activity summary\n- `FitDecoder.decode_and_analyze/1` - Complete workflow in one call","ref":"readme.html#core-functions"},{"type":"extras","title":"Example Helper Function Usage - FitDecoder","doc":"```elixir\n# Get just the activity date\n{:ok, records} = FitDecoder.decode_fit_file_from_path(\"activity.fit\")\n{:ok, date} = FitDecoder.get_activity_date(records)\n# => {:ok, ~D[2024-09-26]}\n\n# Get activity duration\n{:ok, duration} = FitDecoder.get_activity_duration(records)  \n# => {:ok, 1932}  # seconds\n\n# Get comprehensive activity info\n{:ok, info} = FitDecoder.get_activity_info(records)\n# => {:ok, %{\n#      date: ~D[2024-09-26],\n#      duration_seconds: 1932,\n#      total_distance: 1985.8,\n#      record_count: 387,\n#      has_heart_rate: true,\n#      has_altitude: false\n#    }}\n```","ref":"readme.html#example-helper-function-usage"},{"type":"extras","title":"Multi-Session Support - FitDecoder","doc":"The helper functions automatically detect and handle FIT files containing multiple activity sessions, using the longest continuous session for duration and date calculations.","ref":"readme.html#multi-session-support"},{"type":"extras","title":"Installation - FitDecoder","doc":"If [available in Hex](https://hex.pm/docs/publish), the package can be installed by adding `fit_decoder` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:fit_decoder, \"~> 0.1.0\"}\n  ]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Documentation - FitDecoder","doc":"- [Complete Field Reference](FIELDS.md) - All 96+ available fields with descriptions\n- [Field Demo Module](lib/fit_decoder/field_demo.ex) - Interactive exploration of your FIT files\n\nDocumentation can be generated with [ExDoc](https://github.com/elixir-lang/ex_doc) and published on [HexDocs](https://hexdocs.pm). Once published, the docs can be found at  .","ref":"readme.html#documentation"},{"type":"extras","title":"FIT Record Fields Reference","doc":"# FIT Record Fields Reference\n\nThis document lists all the fields available in FIT Record messages that can be extracted using this decoder.","ref":"fields.html"},{"type":"extras","title":"Overview - FIT Record Fields Reference","doc":"Your FIT decoder now supports extracting **96 different fields** from FIT Record messages. These fields cover everything from basic activity data to advanced physiological metrics, diving parameters, and e-bike telemetry.","ref":"fields.html#overview"},{"type":"extras","title":"Field Categories - FIT Record Fields Reference","doc":"","ref":"fields.html#field-categories"},{"type":"extras","title":"Basic Fields - FIT Record Fields Reference","doc":"Always present in records with valid data:\n\n- **`timestamp`** (integer) - Record timestamp in FIT time format\n- **`altitude`** (float) - Altitude in meters\n- **`distance`** (float) - Cumulative distance in meters  \n- **`heart_rate`** (integer) - Heart rate in beats per minute","ref":"fields.html#basic-fields"},{"type":"extras","title":"Position & Navigation - FIT Record Fields Reference","doc":"GPS and movement data:\n\n- **`position_lat`** (integer) - Latitude in semicircles (divide by 2^31 * 180 for degrees)\n- **`position_long`** (integer) - Longitude in semicircles (divide by 2^31 * 180 for degrees)\n- **`enhanced_altitude`** (float) - High-resolution altitude in meters\n- **`speed`** (float) - Speed in meters per second\n- **`enhanced_speed`** (float) - High-resolution speed in m/s\n- **`grade`** (float) - Grade/slope as percentage\n- **`vertical_speed`** (float) - Vertical speed in m/s\n- **`gps_accuracy`** (integer) - GPS accuracy in meters","ref":"fields.html#position-navigation"},{"type":"extras","title":"Power & Performance - FIT Record Fields Reference","doc":"Cycling power and efficiency metrics:\n\n- **`power`** (integer) - Instantaneous power in watts\n- **`accumulated_power`** (integer) - Total accumulated power in watts\n- **`motor_power`** (integer) - E-bike motor power in watts\n- **`left_torque_effectiveness`** (float) - Left leg torque effectiveness percentage\n- **`right_torque_effectiveness`** (float) - Right leg torque effectiveness percentage\n- **`left_pedal_smoothness`** (float) - Left pedal smoothness percentage\n- **`right_pedal_smoothness`** (float) - Right pedal smoothness percentage\n- **`combined_pedal_smoothness`** (float) - Combined pedal smoothness percentage","ref":"fields.html#power-performance"},{"type":"extras","title":"Cadence & Cycling - FIT Record Fields Reference","doc":"Pedaling and cycling-specific metrics:\n\n- **`cadence`** (integer) - Cadence in revolutions per minute\n- **`cadence256`** (float) - High-resolution cadence in RPM\n- **`fractional_cadence`** (float) - Fractional component of cadence\n- **`left_right_balance`** (integer) - Power balance between left/right legs\n- **`cycle_length`** (float) - Length per stroke/pedal cycle\n- **`cycle_length16`** (float) - 16-bit resolution cycle length\n- **`cycles`** (integer) - Number of cycles in this record\n- **`total_cycles`** (integer) - Cumulative total cycles","ref":"fields.html#cadence-cycling"},{"type":"extras","title":"Running Dynamics - FIT Record Fields Reference","doc":"Advanced running form metrics:\n\n- **`vertical_oscillation`** (float) - Vertical oscillation in millimeters\n- **`stance_time`** (float) - Ground contact time in milliseconds\n- **`stance_time_percent`** (float) - Stance time as percentage of stride\n- **`stance_time_balance`** (float) - Left/right ground contact balance\n- **`step_length`** (float) - Step length in millimeters\n- **`vertical_ratio`** (float) - Vertical oscillation to step length ratio","ref":"fields.html#running-dynamics"},{"type":"extras","title":"Physiological Data - FIT Record Fields Reference","doc":"Body metrics and wellness data:\n\n- **`calories`** (integer) - Calories burned\n- **`temperature`** (integer) - Ambient temperature in Celsius\n- **`core_temperature`** (float) - Core body temperature in Celsius\n- **`respiration_rate`** (integer) - Breathing rate in breaths per minute\n- **`enhanced_respiration_rate`** (float) - High-resolution respiration rate\n- **`current_stress`** (float) - Current stress level","ref":"fields.html#physiological-data"},{"type":"extras","title":"Blood/Oxygen Data - FIT Record Fields Reference","doc":"Advanced physiological monitoring:\n\n- **`total_hemoglobin_conc`** (float) - Total hemoglobin concentration\n- **`total_hemoglobin_conc_min`** (float) - Minimum hemoglobin concentration\n- **`total_hemoglobin_conc_max`** (float) - Maximum hemoglobin concentration\n- **`saturated_hemoglobin_percent`** (float) - Blood oxygen saturation (SpO2) percentage\n- **`saturated_hemoglobin_percent_min`** (float) - Minimum SpO2 percentage\n- **`saturated_hemoglobin_percent_max`** (float) - Maximum SpO2 percentage","ref":"fields.html#blood-oxygen-data"},{"type":"extras","title":"E-bike Specific - FIT Record Fields Reference","doc":"Electric bicycle telemetry:\n\n- **`battery_soc`** (float) - Battery state of charge percentage\n- **`ebike_travel_range`** (integer) - Estimated remaining range\n- **`ebike_battery_level`** (integer) - Battery level percentage\n- **`ebike_assist_mode`** (integer) - Current assist mode setting\n- **`ebike_assist_level_percent`** (integer) - Assist level percentage","ref":"fields.html#e-bike-specific"},{"type":"extras","title":"Swimming/Water Sports - FIT Record Fields Reference","doc":"Aquatic activity metrics:\n\n- **`stroke_type`** (integer) - Swimming stroke type (encoded value)\n- **`resistance`** (integer) - Water resistance level\n- **`ball_speed`** (float) - Ball speed for sports applications","ref":"fields.html#swimming-water-sports"},{"type":"extras","title":"Diving - FIT Record Fields Reference","doc":"Scuba diving and underwater metrics:\n\n- **`depth`** (float) - Water depth in meters\n- **`absolute_pressure`** (integer) - Absolute pressure\n- **`next_stop_depth`** (float) - Next decompression stop depth\n- **`next_stop_time`** (integer) - Time to next decompression stop\n- **`time_to_surface`** (integer) - Estimated time to surface\n- **`ndl_time`** (integer) - No decompression limit time remaining\n- **`cns_load`** (integer) - CNS oxygen toxicity load percentage\n- **`n2_load`** (integer) - Nitrogen load for decompression\n- **`air_time_remaining`** (integer) - Remaining air time\n- **`ascent_rate`** (float) - Current ascent rate\n- **`po2`** (float) - Partial pressure of oxygen\n- **`pressure_sac`** (float) - Surface air consumption rate (pressure)\n- **`volume_sac`** (float) - Surface air consumption rate (volume)\n- **`rmv`** (float) - Respiratory minute volume","ref":"fields.html#diving"},{"type":"extras","title":"Other Fields - FIT Record Fields Reference","doc":"Miscellaneous activity data:\n\n- **`activity_type`** (integer) - Type of activity being recorded\n- **`device_index`** (integer) - Index of recording device\n- **`zone`** (integer) - Current training zone\n- **`time128`** (float) - High-resolution timestamp (128Hz)\n- **`grit`** (float) - Grit score (difficulty/intensity metric)\n- **`flow`** (float) - Flow score (performance smoothness)\n- **`time_from_course`** (float) - Time deviation from planned course\n- **`left_pco`** (integer) - Left platform center offset\n- **`right_pco`** (integer) - Right platform center offset","ref":"fields.html#other-fields"},{"type":"extras","title":"Usage Examples - FIT Record Fields Reference","doc":"","ref":"fields.html#usage-examples"},{"type":"extras","title":"Basic Usage - FIT Record Fields Reference","doc":"```elixir\n# Read and decode FIT file\n{:ok, fit_data} = File.read(\"activity.fit\")\nrecords = FitDecoder.decode(fit_data)\n\n# Access basic fields\nEnum.each(records, fn record ->\n  IO.puts(\"Time: #{record.timestamp}\")\n  IO.puts(\"Distance: #{record.distance}m\")\n  if Map.has_key?(record, :heart_rate) do\n    IO.puts(\"HR: #{record.heart_rate} bpm\")\n  end\nend)\n```","ref":"fields.html#basic-usage"},{"type":"extras","title":"Power Analysis - FIT Record Fields Reference","doc":"```elixir\n# Extract power data for cycling analysis\npower_data = records\n|> Enum.filter(&Map.has_key?(&1, :power))\n|> Enum.map(fn record ->\n  %{\n    timestamp: record.timestamp,\n    power: record.power,\n    cadence: Map.get(record, :cadence),\n    heart_rate: Map.get(record, :heart_rate)\n  }\nend)\n\navg_power = power_data \n|> Enum.map(& &1.power) \n|> Enum.sum() \n|> div(length(power_data))\n\nIO.puts(\"Average power: #{avg_power}W\")\n```","ref":"fields.html#power-analysis"},{"type":"extras","title":"GPS Tracking - FIT Record Fields Reference","doc":"```elixir\n# Extract GPS coordinates\ngps_points = records\n|> Enum.filter(fn record ->\n  Map.has_key?(record, :position_lat) && Map.has_key?(record, :position_long)\nend)\n|> Enum.map(fn record ->\n  %{\n    lat: record.position_lat / :math.pow(2, 31) * 180,\n    lng: record.position_long / :math.pow(2, 31) * 180,\n    altitude: Map.get(record, :altitude),\n    timestamp: record.timestamp\n  }\nend)\n```","ref":"fields.html#gps-tracking"},{"type":"extras","title":"Running Dynamics - FIT Record Fields Reference","doc":"```elixir\n# Analyze running form\nrunning_metrics = records\n|> Enum.filter(&Map.has_key?(&1, :vertical_oscillation))\n|> Enum.map(fn record ->\n  %{\n    vertical_oscillation: record.vertical_oscillation,\n    stance_time: Map.get(record, :stance_time),\n    step_length: Map.get(record, :step_length),\n    vertical_ratio: Map.get(record, :vertical_ratio)\n  }\nend)\n```","ref":"fields.html#running-dynamics-1"},{"type":"extras","title":"Advanced Physiological Data - FIT Record Fields Reference","doc":"```elixir\n# Extract comprehensive health metrics\nhealth_data = records\n|> Enum.map(fn record ->\n  %{\n    timestamp: record.timestamp,\n    heart_rate: Map.get(record, :heart_rate),\n    respiration_rate: Map.get(record, :respiration_rate),\n    spo2: Map.get(record, :saturated_hemoglobin_percent),\n    stress: Map.get(record, :current_stress),\n    core_temp: Map.get(record, :core_temperature)\n  }\n  |> Enum.filter(fn {_k, v} -> v != nil end)\n  |> Map.new()\nend)\n```","ref":"fields.html#advanced-physiological-data"},{"type":"extras","title":"Field Availability - FIT Record Fields Reference","doc":"Not all fields will be present in every FIT file. Field availability depends on:\n\n1. **Device capabilities** - Only devices with specific sensors can record certain metrics\n2. **Activity type** - Swimming fields won't appear in cycling activities\n3. **Recording settings** - Some fields may be disabled to save battery/storage\n4. **FIT file version** - Newer fields may not exist in older files\n\nAlways check for field presence using `Map.has_key?/2` before accessing values.","ref":"fields.html#field-availability"},{"type":"extras","title":"Data Types - FIT Record Fields Reference","doc":"- **integer** - Whole numbers (timestamps, counts, enums)\n- **float** - Decimal numbers (distances, speeds, percentages)\n\nInvalid/missing values are filtered out during decoding, so fields are only present when they contain valid data.","ref":"fields.html#data-types"},{"type":"extras","title":"Converting Common Fields - FIT Record Fields Reference","doc":"","ref":"fields.html#converting-common-fields"},{"type":"extras","title":"GPS Coordinates - FIT Record Fields Reference","doc":"```elixir\nlat_degrees = record.position_lat / :math.pow(2, 31) * 180\nlng_degrees = record.position_long / :math.pow(2, 31) * 180\n```","ref":"fields.html#gps-coordinates"},{"type":"extras","title":"Timestamps - FIT Record Fields Reference","doc":"```elixir\n# FIT timestamps are seconds since 1989-12-31 00:00:00 UTC\nfit_epoch = ~U[1989-12-31 00:00:00Z] |> DateTime.to_unix()\nactual_time = DateTime.from_unix!(fit_epoch + record.timestamp)\n```","ref":"fields.html#timestamps"},{"type":"extras","title":"Speed Conversions - FIT Record Fields Reference","doc":"```elixir\n# m/s to km/h\nspeed_kmh = record.speed * 3.6\n\n# m/s to mph  \nspeed_mph = record.speed * 2.237\n```\n\nThis comprehensive field support enables detailed analysis of fitness activities, from basic tracking to advanced performance metrics and specialized applications like diving and e-bike monitoring.","ref":"fields.html#speed-conversions"},{"type":"extras","title":"FitDecoder Helper Functions","doc":"# FitDecoder Helper Functions\n\nThis document describes the helper functions added to the FitDecoder library to simplify common workflows for application developers.","ref":"helper_functions.html"},{"type":"extras","title":"Overview - FitDecoder Helper Functions","doc":"The helper functions provide a streamlined way to:\n1. Decode FIT files from file paths\n2. Extract activity date and duration\n3. Get comprehensive activity information\n4. Handle multi-session FIT files automatically","ref":"helper_functions.html#overview"},{"type":"extras","title":"Functions - FitDecoder Helper Functions","doc":"### `decode_fit_file_from_path/1`\n\nDecodes a FIT file directly from a file path.\n\n**Parameters:**\n- `file_path` (string) - Path to the FIT file\n\n**Returns:**\n- List of decoded records on success\n- `{:error, reason}` on file read error\n- Error atoms from decoder (e.g., `:error_integrity_check_failed`)\n\n**Example:**\n```elixir\ncase FitDecoder.decode_fit_file_from_path(\"/path/to/activity.fit\") do\n  records when is_list(records) ->\n    IO.puts(\"Decoded #{length(records)} records\")\n  {:error, reason} ->\n    IO.puts(\"File error: #{reason}\")\n  error_atom ->\n    IO.puts(\"Decoder error: #{error_atom}\")\nend\n```\n\n### `get_activity_date/1`\n\nExtracts the activity start date from decoded records.\n\n**Parameters:**\n- `records` - List of decoded FIT records\n\n**Returns:**\n- `{:ok, %Date{}}` - Activity start date\n- `{:error, :no_records}` - Empty records list\n- `{:error, :invalid_records}` - No valid timestamps found\n\n**Example:**\n```elixir\n{:ok, date} = FitDecoder.get_activity_date(records)\n# => {:ok, ~D[2024-09-26]}\n```\n\n### `get_activity_duration/1`\n\nCalculates the activity duration from decoded records.\n\n**Parameters:**\n- `records` - List of decoded FIT records\n\n**Returns:**\n- `{:ok, duration_seconds}` - Duration in seconds (integer)\n- `{:error, :no_records}` - Empty records list\n- `{:error, :invalid_records}` - No valid timestamps found\n- `{:error, :insufficient_data}` - Only one record available\n\n**Example:**\n```elixir\n{:ok, duration} = FitDecoder.get_activity_duration(records)\n# => {:ok, 1932}  # 32 minutes 12 seconds\n```\n\n### `get_activity_info/1`\n\nProvides comprehensive activity information in a single call.\n\n**Parameters:**\n- `records` - List of decoded FIT records\n\n**Returns:**\n- `{:ok, activity_info}` - Map containing activity details\n- `{:error, reason}` - If records are invalid or insufficient\n\n**Activity Info Map:**\n```elixir\n%{\n  date: ~D[2024-09-26],           # Activity start date\n  duration_seconds: 1932,         # Duration in seconds\n  total_distance: 1985.8,         # Total distance in meters (nil if not available)\n  record_count: 361,              # Number of records in main session\n  has_heart_rate: true,           # Whether heart rate data is present\n  has_altitude: false             # Whether altitude data is present\n}\n```\n\n**Example:**\n```elixir\n{:ok, info} = FitDecoder.get_activity_info(records)\nIO.puts(\"#{info.duration_seconds} second activity on #{info.date}\")\n```\n\n### `decode_and_analyze/1`\n\nComplete workflow function that combines file reading, decoding, and analysis.\n\n**Parameters:**\n- `file_path` (string) - Path to the FIT file\n\n**Returns:**\n- `{:ok, {activity_info, records}}` - Tuple containing activity info and full records\n- `{:error, reason}` - File read error\n- Error atoms from decoder\n\n**Example:**\n```elixir\ncase FitDecoder.decode_and_analyze(\"/path/to/activity.fit\") do\n  {:ok, {activity_info, records}} ->\n    IO.puts(\"Activity: #{activity_info.duration_seconds}s on #{activity_info.date}\")\n    IO.puts(\"Records available: #{length(records)}\")\n    \n    # Use records for detailed analysis\n    heart_rates = records \n    |> Enum.map(&Map.get(&1, :heart_rate)) \n    |> Enum.filter(&(&1 != nil))\n    \n  {:error, reason} ->\n    IO.puts(\"Error: #{reason}\")\nend\n```","ref":"helper_functions.html#functions"},{"type":"extras","title":"Multi-Session Support - FitDecoder Helper Functions","doc":"The helper functions automatically handle FIT files that contain multiple activity sessions (common in some devices). The functions:\n\n1. **Detect session boundaries** - Identify gaps longer than 1 hour between timestamps\n2. **Select longest session** - Use the session with the longest duration for calculations\n3. **Provide accurate metrics** - Duration and date reflect the main activity session\n\nThis ensures that:\n- Duration calculations are reasonable (e.g., 32 minutes instead of 60 days)\n- Date reflects the actual activity date\n- Statistics are based on the primary activity session","ref":"helper_functions.html#multi-session-support"},{"type":"extras","title":"Error Handling - FitDecoder Helper Functions","doc":"All helper functions use consistent error handling:\n\n- `{:error, :no_records}` - Empty or missing records\n- `{:error, :invalid_records}` - Records without valid timestamps\n- `{:error, :insufficient_data}` - Not enough data for calculation\n- `{:error, file_reason}` - File system errors (e.g., `:enoent`)\n- Error atoms - FIT decoder errors (e.g., `:error_integrity_check_failed`)","ref":"helper_functions.html#error-handling"},{"type":"extras","title":"Recommended Usage Patterns - FitDecoder Helper Functions","doc":"","ref":"helper_functions.html#recommended-usage-patterns"},{"type":"extras","title":"Pattern 1: Quick Analysis - FitDecoder Helper Functions","doc":"```elixir\n# Single function call for most use cases\n{:ok, {info, _records}} = FitDecoder.decode_and_analyze(file_path)\nprocess_activity_summary(info)\n```","ref":"helper_functions.html#pattern-1-quick-analysis"},{"type":"extras","title":"Pattern 2: Detailed Analysis - FitDecoder Helper Functions","doc":"```elixir\n# When you need both summary and detailed record access\n{:ok, {info, records}} = FitDecoder.decode_and_analyze(file_path)\nshow_summary(info)\nanalyze_heart_rate_zones(records)\nplot_elevation_profile(records)\n```","ref":"helper_functions.html#pattern-2-detailed-analysis"},{"type":"extras","title":"Pattern 3: Step-by-Step - FitDecoder Helper Functions","doc":"```elixir\n# When you need granular control\n{:ok, records} = FitDecoder.decode_fit_file_from_path(file_path)\n{:ok, date} = FitDecoder.get_activity_date(records)\n{:ok, duration} = FitDecoder.get_activity_duration(records)\ncustom_analysis(records, date, duration)\n```","ref":"helper_functions.html#pattern-3-step-by-step"},{"type":"extras","title":"Type Specifications - FitDecoder Helper Functions","doc":"```elixir\n@type record :: %{timestamp: integer(), optional(atom()) => term()}\n@type activity_info :: %{\n  date: Date.t(),\n  duration_seconds: integer(),\n  total_distance: float() | nil,\n  record_count: integer(),\n  has_heart_rate: boolean(),\n  has_altitude: boolean()\n}\n\n@spec decode_fit_file_from_path(String.t()) :: [record()] | {:error, term()} | atom()\n@spec get_activity_date([record()]) :: {:ok, Date.t()} | {:error, atom()}\n@spec get_activity_duration([record()]) :: {:ok, integer()} | {:error, atom()}\n@spec get_activity_info([record()]) :: {:ok, activity_info()} | {:error, atom()}\n@spec decode_and_analyze(String.t()) :: {:ok, {activity_info(), [record()]}} | {:error, term()} | atom()\n```","ref":"helper_functions.html#type-specifications"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}